#include "systems.hpp"
#include <fmt/core.h>
#include <string>

using std::string;
using namespace fmt;

using namespace Components;

void System::enemy_pathing(DinkyECS::World &world, Map &game_map, Player &player) {
  const auto &player_position = world.component<Position>(player.entity);
  game_map.set_target(player_position.location);
  game_map.make_paths();

  world.system<Position, Motion>([&](const auto &ent, auto &position, auto &motion) {
    if(ent != player.entity) {
      Point out = position.location;
      game_map.neighbors(out, false);
      motion = { int(out.x - position.location.x), int(out.y - position.location.y)};
    }
  });
  game_map.clear_target(player_position.location);
}


void System::motion(DinkyECS::World &world, Map &game_map) {
  world.system<Position, Motion>([&](const auto &ent, auto &position, auto &motion) {
    Point move_to = {
      position.location.x + motion.dx,
      position.location.y + motion.dy
    };
    motion = {0,0}; // clear it after getting it

    if(game_map.inmap(move_to.x, move_to.y) && !game_map.iswall(move_to.x,move_to.y)) {
      position.location = move_to;
    }
  });
}


void System::combat(DinkyECS::World &world, Player &player) {
  const auto& player_position = world.component<Position>(player.entity);

  world.system<Position, Combat>([&](const auto &ent, auto &pos, auto &combat) {
      if(ent != player.entity && pos.location.x == player_position.location.x &&
          pos.location.y == player_position.location.y) {
      }
  });
};

void System::draw_entities(DinkyECS::World &world, Map &game_map, ftxui::Canvas &canvas, const Point &cam_orig, size_t view_x, size_t view_y) {
  world.system<Position, Tile>([&](const auto &ent, auto &pos, auto &tile) {
    if(pos.location.x >= cam_orig.x && pos.location.x <= cam_orig.x + view_x
        && pos.location.y >= cam_orig.y && pos.location.y <= cam_orig.y + view_y) {
      Point loc = game_map.map_to_camera(pos.location, cam_orig);
      canvas.DrawText(loc.x*2, loc.y*4, tile.chr);
    }
  });
}

void System::draw_map(DinkyECS::World &world, Map &game_map, ftxui::Canvas &canvas, size_t view_x, size_t view_y) {
  const auto& player = world.get<Player>();
  const auto& player_position = world.component<Position>(player.entity);
  Point start = game_map.center_camera(player_position.location, view_x, view_y);
  Matrix &walls = game_map.walls();

  size_t end_x = std::min(view_x, game_map.width() - start.x);
  size_t end_y = std::min(view_y, game_map.height() - start.y);

  for(size_t x = 0; x < end_x; ++x) {
    for(size_t y = 0; y < end_y; ++y) {
      string tile = walls[start.y+y][start.x+x] == 1 ? WALL_TILE : FLOOR_TILE;
      canvas.DrawText(x * 2, y * 4, tile);
    }
  }

  System::draw_entities(world, game_map, canvas, start, view_x, view_y);
}
